/// Custom on-demand processor
/// - issues metadata update for changed serviceTime
/// - adds supports for multiple (C62) documents
/// 
Class HS.Local.VA.HS.Registry.HS.IHE.XDSb.OnDemand.Operations Extends HS.IHE.XDSb.OnDemand.Operations
{

/// CUSTOM: override, quit if all docs removed
/// 
Method deliverMessage(pMetadata As HS.IHE.XDSb.Types.Metadata, pRequest As HS.Message.XMLMessage, ByRef pResponse As HS.Message.XMLMessage) As %Status
{
	Set tSC = $$$OK
	Try {
		$$$HSPerfStart("Check prior submissions")
			Set tIsOnDemand = (pRequest.AdditionalInfo.GetAt("DeliveryType") = "XDSb.OnDemand")
		
			#; OnDemand replacement can be skipped since the metadata will not change,
			#; but the capability is required for IHE certification and may be useful someday
			Set tSC = ..setPrevious(pMetadata,.tHasPrevious) $$$ThrowOnError(tSC)
			If tHasPrevious {
				If tIsOnDemand, '..IssueOnDemandReplacements {
					$$$HSPerfEnd
					$$$HSTRACE("Skipping on demand replacement")
					Quit
				}
				$$$ThrowOnError(..checkPrevious(pMetadata))
			}
		$$$HSPerfEnd

		#; CUSTOM: nothing to do if all documents were removed by 'checkPrevious'
		Quit:pMetadata.Documents.Count()=0
		
		#; Determine the message type (register for on-demand, provide for stable)
		$$$HSPerfStart("Create XDSb Message")
			If tIsOnDemand {
				Set tParams("messageName") = $$$XDSbRegisterOnDemandRequest

				#; Set the repository for each on-demand document
				Set tThisRepositoryOID = ##class(HS.Data.OIDMap).OIDForType(..OnDemandRepositoryName,"Repository")
				Set i="" For { Set tDoc=pMetadata.Documents.GetNext(.i) Quit:i=""  
					Set tDoc.RepositoryUniqueId = tThisRepositoryOID
				}
			}
			Else {		
				Set tParams("messageName") = $$$XDSbProvideAndRegisterRequest
			}
			#; Create and send the XDSb message		
			Set tSC = ..TransformObjects(pMetadata, ..MessageTransform, .tRequest, .tParams, "HS.Message.XMLMessage",,1) $$$ThrowOnError(tSC)
			Set tSC = ##class(HS.IHE.Util).MergeAdditionalInfo(pRequest, tRequest) $$$ThrowOnError(tSC)
			Set tRequest.StreamCollection = pRequest.StreamCollection
		$$$HSPerfEnd
		
		$$$HSPerfStart("Register or Provide")		
			Set tSC = ..OnMessage(tRequest, .pResponse) $$$ThrowOnError(tSC)
		$$$HSPerfEnd
		
		#; Update submissions if successful
		$$$HSPerfStart("Update Submissions")		
			Set tSC = ..updateSubmissions(pMetadata) $$$ThrowOnError(tSC)
		$$$HSPerfEnd
		
	} Catch ex { Set tSC = ex.AsStatus() }
	Quit tSC
}

/// CUSTOM: override to remove documents (and their associations) that have changes we don't care about
/// 
Method checkPrevious(pMetadata As HS.IHE.XDSb.Types.Metadata) As %Status
{
	Set tSC = $$$OK
	Try {
		#dim tAssoc As HS.IHE.XDSb.Types.Association
		#dim tPrevious As HS.IHE.XDSb.Types.Document
		#dim tOutputList As HS.IHE.XDSb.Types.Metadata
		
		#; Build a list of previous documents
		Set tInputList = ##class(HS.IHE.XDSb.Types.Metadata).%New()
		Set i="" For { Set tAssoc = pMetadata.Associations.GetNext(.i) Quit:i=""
			Continue:tAssoc.Type'="Replaces"
			If tAssoc.ChildUUID]"" {
				Set tPrevious = ##class(HS.IHE.XDSb.Types.Document).%New()
				Set tPrevious.EntryUUID = tAssoc.ChildUUID
				Do tInputList.Documents.Insert(tPrevious)
			}
		}
		If tInputList.Documents.Count() = 0 Quit
		
		#; Query registry for previous document status
		Set tSC = ..queryGetDocuments(tInputList, .tOutputList) $$$ThrowOnError(tSC)
		
		#; Clean up if previous document is no longer approved
		Set i="" For { Set tPrevious = tOutputList.Documents.GetNext(.i) Quit:i=""
			If tPrevious.AvailabilityStatus '= "Approved" {
				Set j="" For { Set tAssoc = pMetadata.Associations.GetPrevious(.j) Quit:j=""
					If tAssoc.ChildUUID = tPrevious.EntryUUID {
						#; Remove the replace association
						Do pMetadata.Associations.RemoveAt(j)
						
						#; Remove entry from submission tracking table
						#dim tResult As HS.Util.WebServices.ServiceStatus
						Set tSubmission = ##class(HS.Push.WebServices.Containers.Submission).%New()
						#; CUSTOM: fix - should be ChildUUID
						#;Set tSubmission.DocumentId = tAssoc.ParentUUID
						Set tSubmission.DocumentId = tAssoc.ChildUUID
						$$$HSTRACE("Calling DeleteSubmission","tSubmission",tSubmission)
						Set tSC = ##class(HS.Util.HubServices).InvokeHubService("HS.Push.WebServices","HS.Push.WebServicesClient","DeleteSubmission",.tResult,tSubmission)
						$$$ThrowOnError(tSC)
						$$$ThrowOnError(tResult.Status)
					}
				}
			}
			Else {
				#; CUSTOM: remove documents/associations unless key fields changed
				#;
				Set j="" For { Set tAssoc = pMetadata.Associations.GetPrevious(.j) Quit:j=""
					Continue:tAssoc.Type'="Replaces"
					Continue:tAssoc.ChildUUID'=tPrevious.EntryUUID
					Set k="" For { Set tDocument = pMetadata.Documents.GetPrevious(.k) Quit:k=""
						Continue:tDocument.EntryUUID'=tAssoc.ParentUUID
						Continue:(tDocument.ServiceStartTime'=tPrevious.ServiceStartTime)
						Continue:(tDocument.ServiceStopTime'=tPrevious.ServiceStopTime)
						Set tRemoved(tDocument.EntryUUID) = ""
						Do pMetadata.Documents.RemoveAt(k)
						Do pMetadata.Associations.RemoveAt(j)
					}
				}
				#; second pass to remove any associations tied to a removed document (eg HasMember)
				Set tUUID="" For { Set tUUID = $O(tRemoved(tUUID)) Quit:tUUID=""
					Set j="" For { Set tAssoc = pMetadata.Associations.GetPrevious(.j) Quit:j=""
						If ((tAssoc.ChildUUID=tUUID)||(tAssoc.ParentUUID=tUUID)) {
							Do pMetadata.Associations.RemoveAt(j)
						}
					}
				}
			}
		}
		
	} Catch ex { Set tSC = ex.AsStatus() }
	Quit tSC
}

/// CUSTOM: override to include fields to check for replacement
/// 
Method queryGetDocuments(pInput As HS.IHE.XDSb.Types.Metadata, Output pOutput As HS.IHE.XDSb.Types.Metadata) As %Status
{
	#dim tDocument As HS.IHE.XDSb.Types.Document
	#; When on the hub, only used by checkPrevious, which only sends EntryUUID and needs AvailabilityStatus and EntryUUID
	Set tSC = $$$OK
	Set pOutput = ##class(HS.IHE.XDSb.Types.Metadata).%New()
	Try {
		Set i="" For { Set tDocument=pInput.Documents.GetNext(.i) Quit:i=""
			If ##class(HS.Registry.Document).IIDIndexExists(tDocument.EntryUUID,.tID) {
				#dim tDoc As HS.Registry.Document
				Set tDoc = ##class(HS.Registry.Document).%OpenId(tID)
				Set tDocument.AvailabilityStatus = tDoc.Status
				Set tDocument.ServiceStartTime = tDoc.ServiceStartTime
				Set tDocument.ServiceStopTime = tDoc.ServiceStopTime
				Do pOutput.Documents.Insert(tDocument)				
			} Else {
				$$$ThrowStatus($$$ERROR($$$GeneralError,"Document does not exist: "_tDocument.EntryUUID))
			}
		}
	} Catch ex { Set tSC = ex.AsStatus() }
	$$$HSTRACE("Query Result","pOutput",pOutput)
	Quit tSC
}

}
