Include HS.Common

/// PREREQ
/// 
///   1. HealthShare kit installed
///   2. Custom classes are loaded and compiled into HSCUSTOM
///   3. Custom XSLT is copied into appropriate locations
/// 
/// INSTALL
/// 
///   zn "HSCUSTOM"
///   Do ##class(Custom.RegsitrySetup).Install()
/// 
/// UNINSTALL
/// 
///   zn "HSLIB"
///   Do ##class(HS.Util.Installer).UnInstallAll()
/// 
Class HS.Local.VA.HS.Registry.Setup Extends %RegisteredObject
{

ClassMethod Install()
{
	Try {
		Do ..Validate()
		Do ..BaseSystem()
		Do ..MapCustomPackage()
		Do ..CodedEntries()
		Do ..InfoTypes()
		Do ..OIDs()
		Do ..ConfigRegistry()
		Do ..FacilityRegistry()
		Do ..ServiceRegistry()
		Do ..XMLSummary()
		Do ..Delivery()
		Do ..Productions()
	}
	Catch ex {
		Do ex.OutputToDevice()
	}
}

/// Install a base HS environment
/// 
ClassMethod BaseSystem()
{
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:BaseSystem - Installing HealthShare"

	Set options("HubAuditProduction")=1
	Set options("OnDemand")=1
	Set options("Push")=1
	Set options("Trace")=1
	Set options("XDSb.Push")=1
	Set options("XDSb.Registry")=1
	Set options("XDSb.Repository")=1
	$$$ThrowOnError(##class(HS.Local.VA.HS.Registry.Installer).Install(.options))

	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:BaseSystem - success"
}

/// Run after HS any gateway install to ensure custom code is visible to all components
/// 
ClassMethod MapCustomPackage()
{
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:MapCustomPackage - Mapping custom package to all gateways"

	New $namespace
	Set $namespace = "HSCUSTOM"
	Set ns="" For { Set ns=$o(^%SYS("HealthShare","Instances",ns)) Quit:ns=""
		If $$$WriteProg Write !,?4,ns
		Do ##class(HS.Util.SystemAPI).AddClassMappings(ns,"Custom","HSCUSTOM")
		Do ##class(HS.Util.SystemAPI).ActivateConfiguration(ns)
	}

	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:MapCustomPackage - success"
}

/// Setup coded entries required by XDS registry metadata validation
///  
ClassMethod CodedEntries()
{
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:CodedEntries - adding LOINC codes for XDS"

	New $namespace
	Set $namespace = "HSREGISTRY"

	#; LOINC codes
	Set oid = "2.16.840.1.113883.6.1"
	Set codes("34133-9") = "Patient Summary"
	Set codes("11506-3") = "Encounter Summary"
	Set codes("28570-0") = "Provider-Unspecified Procedure Note"
	Set codes("11488-4") = "Consult Note"
	Set codes("18842-5") = "Discharge Summary"
	Set codes("34117-2") = "History and Physical"
	Set codes("27898-6") = "Pathology Study"
	Set codes("18733-6") = "Progress Note"
	Set codes("18733-6") = "Progress Note"
	Set codes("18726-0") = "Radiology Study"
	Set codes("29752-3") = "Perioperative Report"
	For type = "contentTypeCode","classCode","typeCode" {
		If $$$WriteProg Write !,?4,type
		Set code="" For { Set code = $O(codes(code),1,desc) Quit:code=""
			If $$$WriteProg Write !,?8,oid," ",code," ",desc
			$$$ThrowOnError(##class(HS.IHE.CodedEntry).Create(type,oid,code,desc))	
		}
	}
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:CodedEntries - success"
}

/// Prepopulate "seen" document subtypes so they can be specified in push filter
/// 
ClassMethod InfoTypes()
{
	New $namespace
	Set $namespace = "HSREGISTRY"
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:InfoTypes - adding document types"
	
	#; document type codes used for push
	For type="CP","CR","DS","HP","LR","PN","RA","SR","PROGRESS NOTE" {
		If $$$WriteProg Write !,?4,"DOC.",type
		Set ^HS.InfoTypesSub("DOC."_type)=""
	}
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:InfoTypes - success"
}

/// Setup OIDs
/// 
ClassMethod OIDs()
{
	New $namespace
	Set $namespace = "HSREGISTRY"
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:OIDs - adding OIDs"

	If $$$WriteProg Write !,?4,"USVHA - 2.16.840.1.113883.4.349"
	$$$ThrowOnError(##class(HS.Data.OIDMap).FileOID("USVHA","CodeSystem","2.16.840.1.113883.4.349","Department of Veterans Affairs","AssigningAuthority"))

	If $$$WriteProg Write !,?4,"VA - 2.16.840.1.113883.4.349.1"
	$$$ThrowOnError(##class(HS.Data.OIDMap).FileOID("VA","HomeCommunity","2.16.840.1.113883.4.349.1","Department of Veterans Affairs","AssigningAuthority"))
	
	; need to add OIDs for each Edge.  Modify below lines to accomodate Edges connected to your Access Gateway
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:OIDs - adding OIDs for each Edge"
	If $$$WriteProg Write !,?4,"Edges - 1.2.9999999"
	$$$ThrowOnError(##class(HS.Data.OIDMap).FileOID("983","Facility","2.16.840.1.113883.4.349.983","Cheyenne Veterans Affairs Hospital","AssigningAuthority"))
	$$$ThrowOnError(##class(HS.Data.OIDMap).FileOID("984","Facility","2.16.840.1.113883.4.349.984","Dayton Veterans Affairs Hospital","AssigningAuthority"))
	
	; OIDs for our repositories.
	$$$ThrowOnError(##class(HS.Data.OIDMap).FileOID("HSACCCDA", "Repository", "2.16.840.1.113883.4.349.1.1", "On-Demand Repository"))
	$$$ThrowOnError(##class(HS.Data.OIDMap).FileOID("HSREPOSITORY", "Repository", "2.16.840.1.113883.4.349.1.2", "Stable Repository"))
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:OIDs - success"
}

/// Setup configuration registry items
/// 
ClassMethod ConfigRegistry()
{
	New $namespace
	Set $namespace = "HSREGISTRY"
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:ConfigRegistry - updating configuration registry"

	#; IHE domain 
	Set data("\IHE\AffinityDomain") = "VA"
	Set data("\IHE\HomeCommunity") = "VA"

	#; used by CDA transforms to populate custodian
	Set data("\General\IHEVersion") = "1"
	Set data("\HomeCommunity\Address\City") = "Washington"
	Set data("\HomeCommunity\Address\Country") = "USA"
	Set data("\HomeCommunity\Address\State") = "DC"
	Set data("\HomeCommunity\Address\StreetLine1") = "810 Vermont Avenue, NW"
	Set data("\HomeCommunity\Address\Zip") = "20420"
	Set data("\HomeCommunity\Telecom\Workphone") = "(123) 456-7890"
	
	#; TODO - what should this be for the VA patient/encounter doc, the C62 does not use it
	Set data("\LegalAuthenticator\Address\City") = "Washington"
	Set data("\LegalAuthenticator\Address\Country") = "USA"
	Set data("\LegalAuthenticator\Address\State") = "DC"
	Set data("\LegalAuthenticator\Address\StreetLine1") = "810 Vermont Avenue, NW"
	Set data("\LegalAuthenticator\Address\Zip") = "20420"
	Set data("\LegalAuthenticator\Name\Family") = "InterSystems"
	Set data("\LegalAuthenticator\Name\Given") = "HealthShare"
	Set data("\LegalAuthenticator\Telecom\Workphone") = "(123) 456-7890"
	
	Set key="" For { Set key = $o(data(key),1,value) Quit:key=""
		If $$$WriteProg Write !,?4,key," = ",value
		$$$ThrowOnError(##class(HS.Registry.Config).AddUpdateKeyValue(key,value))
	}

	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:ConfigRegistry - success"
}

/// Setup Facilties
/// 
ClassMethod FacilityRegistry()
{
	new $namespace
	set $namespace = "HSREGISTRY"
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:FaciiltyRegistry - start"
	
	set fr = ##class(HS.Facility.Config).%New()
	set fr.FacilityCode = "EF"
	set fr.Name = "EF"
	set fr.Description = "Enterprise Facility"
	set fr.Tier = "1"
	$$$ThrowOnError(fr.%Save())
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:ServiceRegistry - success"
}

/// Setup service Registry
/// 
ClassMethod ServiceRegistry()
{
	New $namespace
	Set $namespace = "HSREGISTRY"
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:ServiceRegistry - start"

	#; HS.Util.Installer.Kit.IHE.XDSb.Registry marks the wrong service as the XDSb.Query device,
	#; which is interfering with the "End Point" drop down in push system delivery policy setup
	#dim service as HS.Registry.Service.SOAP
	If $$$WriteProg Write !,?4,"Fixing XDSb.Registry service..."
	Set service = ##class(HS.Registry.Service.SOAP).EndPointForNameType("XDSb.Registry","SOAP",.sc) $$$ThrowOnError(sc)
	Set service.DeviceFunction = ""
	Set service.Push = 1
	$$$ThrowOnError(service.%Save())
	If $$$WriteProg Write !,?4,"Fixing XDSb.Query service..."
	Set service = ##class(HS.Registry.Service.SOAP).EndPointForNameType("XDSb.Query","SOAP",.sc) $$$ThrowOnError(sc)
	Set service.DeviceFunction = "XDSb.Query"
	Set service.Push = 0
	$$$ThrowOnError(service.%Save())
	
	set service = ##class(HS.Registry.Service.SOAP).%New()
	set service.Name = "MVI.PDQv3.Consumer"
	set service.Host = "MOCK-HOST"
	set service.Port = "8082"
	set service.URL = "/psim_webservice/IdMWebService"
	SET service.SOAPVersion = "1.1"
	$$$ThrowOnError(service.%Save())	
	
	set service = ##class(HS.Registry.Service.SOAP).%New()
	set service.Name = "VA.ESR"
	set service.Host = "MOCK-HOST"
	set service.Port = "8081"
	set service.URL = "/esr-ws/spring-ws/getEESummary/eeSummary"
	SET service.SOAPVersion = "1.1"
	$$$ThrowOnError(service.%Save())

	set service = ##class(HS.Registry.Service.SOAP).%New()
	set service.Name = "VA.CTService"
	set service.Host = "MOCK-HOST"
	set service.Port = "8084"
	set service.URL = "/sts.webservice/ctService"
	set service.SOAPVersion = "1.1"
	$$$ThrowOnError(service.%Save())

	If $$$WriteProg Write "OK"

	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:ServiceRegistry - success"
}

/// Setup XML Summaries
/// 
ClassMethod XMLSummary()
{
	New $namespace
	Set $namespace = "HSREGISTRY"
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:XMLSummary - adding XML summaries"

	Set items($i(items),"Code") = "Z-CCDA-Patient"
	Set items(items,"Description") = "VA Custom Patient Summary (CCDA)"
	Set items(items,"SDAVersion") = 3
	Set items(items,"FormatCode") = "urn:hl7-org:sdwg:ccda-structuredBody:1.1"
	Set items(items,"FormatScheme") = "1.3.6.1.4.1.19376.1.2.3"
	Set items(items,"SectionTypes") = $LB("PAT","ADV","ALG","APT","DXG","ENC","PRB","HIS","LAB","OBS","RAD","PRC","PEX","MED","VXU","DOC")
	
	Set items($i(items),"Code") = "Z-CCDA-Encounter"
	Set items(items,"Description") = "VA Custom Encounter Summary (CCDA)"
	Set items(items,"SDAVersion") = 3
	Set items(items,"FormatCode") = "urn:hl7-org:sdwg:ccda-structuredBody:2.1"
	Set items(items,"FormatScheme") = "1.3.6.1.4.1.19376.1.2.3"
	Set items(items,"SectionTypes") = $LB("PAT","ADV","ALG","APT","DXG","ENC","PRB","HIS","LAB","OBS","RAD","PRC","PEX","MED","VXU","DOC")

	Set items($i(items),"Code") = "Z-C62"
	Set items(items,"Description") = "VA Custom Unstructured Document (C62)"
	Set items(items,"SDAVersion") = 3
	Set items(items,"FormatCode") = "urn:ihe:iti:xds-sd:text:2008"
	Set items(items,"FormatScheme") = "1.3.6.1.4.1.19376.1.2.3"
	Set items(items,"SectionTypes") = $LB("PAT","ENC","DOC")
	
	Set items($i(items),"Code") = "Z-VA-CCDAv21-CCD"
	Set items(items,"Description") = "VA Custom Patient Summary (CCDA) v2.1"
	Set items(items,"SDAVersion") = 3
	Set items(items,"FormatCode") = "urn:hl7-org:sdwg:ccda-structuredBody:2.1"
	Set items(items,"FormatScheme") = "1.3.6.1.4.1.19376.1.2.3"
	Set items(items,"SectionTypes") = $LB("PAT","ADV","ALG","APT","DXG","ENC","PRB","HIS","LAB","OBS","RAD","PRC","PEX","MED","VXU","DOC")
	
	Set items($i(items),"Code") = "Z-C32v25"
	Set items(items,"Description") = "VA Custom Continuity of Care Document (CCD, HITSP C32)"
	Set items(items,"SDAVersion") = 3
	Set items(items,"FormatCode") = "urn:ihe:pcc:xphr:2007"
	Set items(items,"FormatScheme") = "IHE PCC"
	Set items(items,"SectionTypes") = $LB("PAT","ALG","ENC","PRB","LAB","OBS","PRC","MED","VXU","DOC")
	
	Set items($i(items),"Code") = "Z-MHV-CCDA-CCD"
	Set items(items,"Description") = "VA Custom MyHealth eVet Custom Patient Summary (CCDA)"
	Set items(items,"SDAVersion") = 3
	Set items(items,"FormatCode") = "urn:hl7-org:sdwg:ccda-structuredBody:1.1"
	Set items(items,"FormatScheme") = "1.3.6.1.4.1.19376.1.2.3"
	Set items(items,"SectionTypes") = $LB("PAT","ADV","ALG","APT","DXG","ENC","PRB","HIS","LAB","OBS","RAD","PRC","PEX","MED","VXU","DOC")
	
	Set items($i(items),"Code") = "Z-MHV-CCDAv21-CCD"
	Set items(items,"Description") = "VA Custom MyHealth eVet Custom Patient Summary (CCDA) v2.1"
	Set items(items,"SDAVersion") = 3
	Set items(items,"FormatCode") = "urn:hl7-org:sdwg:ccda-structuredBody:2.1"
	Set items(items,"FormatScheme") = "1.3.6.1.4.1.19376.1.2.3"
	Set items(items,"SectionTypes") = $LB("PAT","ADV","ALG","APT","DXG","ENC","PRB","HIS","LAB","OBS","RAD","PRC","PEX","MED","VXU","DOC")
	
	For i = 1:1:items {
		If $$$WriteProg Write !,?4,items(i,"Code")
		Set item = ##class(HS.Reports.Patient.XMLSummaryType).%New()
		Set item.Code = $G(items(i,"Code"))
		Set item.Description = $G(items(i,"Description"))
		Set item.ResourceRequired = $G(items(i,"ResourceRequired"))
		Set item.SDAVersion = $G(items(i,"SDAVersion"))
		Set item.FormatCode = $G(items(i,"FormatCode"))
		Set item.FormatScheme = $G(items(i,"FormatScheme"))
		For j=1:1:$LL(items(i,"SectionTypes")) { 
			Do item.SectionTypes.Insert($LI(items(i,"SectionTypes"),j))
		}
		$$$ThrowOnError(##class(HS.Util.HubServices).InvokeHubService("HS.Hub.HSWS.WebServices","HS.Hub.HSWS.WebServicesClient","AddEditXMLSummaryType",.sc,item))
		$$$ThrowOnError(sc)
	}
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:XMLSummary - success"
}

/// Setup delivery for on-demands
/// 
ClassMethod Delivery()
{
	New $namespace
	Set $namespace = "HSREGISTRY"
	
	Do ##class(HS.Push.Subscription.SystemPolicy).%DeleteExtent()
	Do ##class(HS.Push.Delivery.SystemPolicy).%DeleteExtent()
	Do ##class(HS.Push.Registry.Recipient).%DeleteExtent()
	Do ##class(HS.Push.DeliveryOption).%DeleteExtent()
	Do ##class(HS.Push.Transformation).%DeleteExtent()
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:XMLSummary - start"
	
	If $$$WriteProg Write !,?4,"Push Transforms"
	
	Set transform = ##class(HS.Push.WebServices.Containers.Transformation).%New()
	Set transform.Name               = "CCDA Patient Summary"
	Set transform.ContentScope       = "MPI"
	Set transform.TransformationType = "XSLT"
	Set transform.XSLTFileSpec       = "Z-CCDA-Patient"
	Set txPatientCCDA = $$addTransform(transform)
	
	; Add for each on-demand document format you need
	Set transform = ##class(HS.Push.WebServices.Containers.Transformation).%New()
	Set transform.Name               = "CCDA v2.1 Patient Summary"
	Set transform.ContentScope       = "MPI"
	Set transform.TransformationType = "XSLT"
	Set transform.XSLTFileSpec       = "Z-VA-CCDAv21-CCD"
	Set txPatientCCDA21 = $$addTransform(transform)
	
	Set transform = ##class(HS.Push.WebServices.Containers.Transformation).%New()
	Set transform.Name               = "MHV CCDA Patient Summary"
	Set transform.ContentScope       = "MPI"
	Set transform.TransformationType = "XSLT"
	Set transform.XSLTFileSpec       = "Z-MHV-CCDA-CCD"
	Set txPatientMHVCCDA = $$addTransform(transform)
	
	Set transform = ##class(HS.Push.WebServices.Containers.Transformation).%New()
	Set transform.Name               = "MHV CCDA v2.1 Patient Summary"
	Set transform.ContentScope       = "MPI"
	Set transform.TransformationType = "XSLT"
	Set transform.XSLTFileSpec       = "Z-MHV-CCDAv21-CCD"
	Set txPatientMHVCCDA21 = $$addTransform(transform)
	
	Set transform = ##class(HS.Push.WebServices.Containers.Transformation).%New()
	Set transform.Name               = "C32 Patient Summary"
	Set transform.ContentScope       = "MPI"
	Set transform.TransformationType = "XSLT"
	Set transform.XSLTFileSpec       = "Z-C32v25"
	Set txPatientC32 = $$addTransform(transform)
	
	Set transform = ##class(HS.Push.WebServices.Containers.Transformation).%New()
	Set transform.Name               = "CCDA Encounter Summary"
	Set transform.ContentScope       = "Enc"
	Set transform.TransformationType = "XSLT"
	Set transform.XSLTFileSpec       = "Z-CCDA-Encounter"
	Set txEncounter = $$addTransform(transform)
	
	Set transform = ##class(HS.Push.WebServices.Containers.Transformation).%New()
	Set transform.Name               = "C62 Unstructured Document"
	Set transform.ContentScope       = "Enc"
	Set transform.TransformationType = "XSLT"
	Set transform.XSLTFileSpec       = "Z-C62"
	Set txDocument = $$addTransform(transform)
	
	If $$$WriteProg Write !,?4,"Delivery Options"
	
	Set option = ##class(HS.Push.WebServices.Containers.DeliveryOption).%New()
	Set option.Name              = "XDSb OnDemand"
	Set option.DeliveryType      = "XDSb.OnDemand"
	Set option.DeliveryOperation = "HS.IHE.XDSb.Push.Operations"
	Set optOnDemand = $$addOption(option)

	If $$$WriteProg Write !,?4,"Receiving System"
	
	Set system = ##class(HS.Push.WebServices.Containers.Recipient).%New()
	Set system.Name        = "Default"
	Set system.Description = "Internal push (e.g. XDSb on-demand registration)"
	Set system.Active      = 1
	Set sysDefault = $$addSystem(system)
	
	Set policy = ##class(HS.Push.WebServices.Containers.DeliveryPolicy).%New()
	Set policy.RecipientType  = "system"
	Set policy.RegistryId     = sysDefault
	Set policy.Name           = "Register OnDemand"
	Set policy.DeliveryOption = optOnDemand
	Set policy.Transformation = txPatientCCDA21
	;Set policy.AccessGateway  = ..getGatewayID("HSACCESS")
	Set policy.EndPoint       = ..getServiceID("XDSb.Registry")
	Set polOnDemand = $$addPolicy(policy)
	
	Set subscription = ##class(HS.Push.WebServices.Containers.SubscriptionPolicy).%New()
	Set subscription.RecipientType     = "system"
	Set subscription.RegistryId        = sysDefault
	Set subscription.Name              = "OnDemand Patient Summary"
	Set subscription.SubscriptionBasis = "A"  //All
	Set subscription.DeliveryPolicy    = polOnDemand
	Set subscription.Transformation    = txPatientCCDA
	Set subscription.Subject           = "Patient Summary"
	Set subscription.EffectiveDate     = +$H
	Set subscription.ExpirationDate    = ""
	Set subPatient = $$addSubscription(subscription)
	
	; add subscription for each registered document/format
	Set subscription = ##class(HS.Push.WebServices.Containers.SubscriptionPolicy).%New()
	Set subscription.RecipientType     = "system"
	Set subscription.RegistryId        = sysDefault
	Set subscription.Name              = "OnDemand v2.1 Patient Summary"
	Set subscription.SubscriptionBasis = "A"  //All
	Set subscription.DeliveryPolicy    = polOnDemand
	Set subscription.Transformation    = txPatientCCDA21
	Set subscription.Subject           = "Patient Summary 2.1"
	Set subscription.EffectiveDate     = +$H
	Set subscription.ExpirationDate    = ""
	Set subPatient = $$addSubscription(subscription)
	
	;Set subscription = ##class(HS.Push.WebServices.Containers.SubscriptionPolicy).%New()
	;Set subscription.RecipientType     = "system"
	;Set subscription.RegistryId        = sysDefault
	;Set subscription.Name              = "MHV OnDemand Patient Summary"
	;Set subscription.SubscriptionBasis = "A"  //All
	;Set subscription.DeliveryPolicy    = polOnDemand
	;Set subscription.Transformation    = txPatientMHVCCDA
	;Set subscription.Subject           = "MHV Patient Summary"
	;Set subscription.EffectiveDate     = +$H
	;Set subscription.ExpirationDate    = ""
	;Set subPatient = $$addSubscription(subscription)
	
	;Set subscription = ##class(HS.Push.WebServices.Containers.SubscriptionPolicy).%New()
	;Set subscription.RecipientType     = "system"
	;Set subscription.RegistryId        = sysDefault
	;Set subscription.Name              = "MHV OnDemand v2.1 Patient Summary"
	;Set subscription.SubscriptionBasis = "A"  //All
	;Set subscription.DeliveryPolicy    = polOnDemand
	;Set subscription.Transformation    = txPatientMHVCCDA21
	;Set subscription.Subject           = "MHV Patient Summary 2.1"
	;Set subscription.EffectiveDate     = +$H
	;Set subscription.ExpirationDate    = ""
	;Set subPatient = $$addSubscription(subscription)
	
	Set subscription = ##class(HS.Push.WebServices.Containers.SubscriptionPolicy).%New()
	Set subscription.RecipientType     = "system"
	Set subscription.RegistryId        = sysDefault
	Set subscription.Name              = "OnDemand Patient C32"
	Set subscription.SubscriptionBasis = "A"  //All
	Set subscription.DeliveryPolicy    = polOnDemand
	Set subscription.Transformation    = txPatientC32
	Set subscription.Subject           = "Patient Summary"
	Set subscription.EffectiveDate     = +$H
	Set subscription.ExpirationDate    = ""
	Set subPatient = $$addSubscription(subscription)

	Set subscription = ##class(HS.Push.WebServices.Containers.SubscriptionPolicy).%New()
	Set subscription.RecipientType     = "system"
	Set subscription.RegistryId        = sysDefault
	Set subscription.Name              = "OnDemand Encounter Summary"
	Set subscription.SubscriptionBasis = "I"  //Info Type
	Set subscription.DeliveryPolicy    = polOnDemand
	Set subscription.Transformation    = txEncounter
	Set subscription.Subject           = "Encounter Summary"
	Set subscription.EffectiveDate     = +$H
	Set subscription.ExpirationDate    = ""
	Set subEncounter = $$addSubscription(subscription)
	Do addSearchTerms(subEncounter,subscription.SubscriptionBasis,$LB("ENC"))

	Set subscription = ##class(HS.Push.WebServices.Containers.SubscriptionPolicy).%New()
	Set subscription.RecipientType     = "system"
	Set subscription.RegistryId        = sysDefault
	Set subscription.Name              = "OnDemand Unstructured Document"
	Set subscription.SubscriptionBasis = "I"  //Info Type
	Set subscription.DeliveryPolicy    = polOnDemand
	Set subscription.Transformation    = txDocument
	Set subscription.Subject           = "Unstructured Document"
	Set subscription.EffectiveDate     = +$H
	Set subscription.ExpirationDate    = ""
	Set subDocument = $$addSubscription(subscription)
	Do addSearchTerms(subDocument,subscription.SubscriptionBasis,$LB("DOC.CP","DOC.CR","DOC.DS","DOC.HP","DOC.LR","DOC.PN","DOC.RA","DOC.SR","DOC.PROGRESS NOTE"))
	
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:XMLSummary - success"
	Quit
	
addTransform(transform)
	If $$$WriteProg Write !,?8,transform.Name
	$$$ThrowOnError(##class(HS.Util.HubServices).InvokeHubService("HS.Push.WebServices","HS.Push.WebServicesClient","AddEditTransformation",.response,transform))
	$$$ThrowOnError(response.Status)
	Quit response.ID
addOption(option)
	If $$$WriteProg Write !,?8,option.Name
	$$$ThrowOnError(##class(HS.Util.HubServices).InvokeHubService("HS.Push.WebServices","HS.Push.WebServicesClient","AddEditDeliveryOption",.response,option))
	$$$ThrowOnError(response.Status)
	Quit response.ID
addSystem(system)
	If $$$WriteProg Write !,?8,system.Name
	$$$ThrowOnError(##class(HS.Util.HubServices).InvokeHubService("HS.Push.WebServices","HS.Push.WebServicesClient","AddEditReceivingSystem",.response,system))
	$$$ThrowOnError(response.Status)
	Quit response.ID
addPolicy(policy)
	If $$$WriteProg Write !,?12,"Delivery Policy: ",policy.Name
	$$$ThrowOnError(##class(HS.Util.HubServices).InvokeHubService("HS.Push.WebServices","HS.Push.WebServicesClient","AddEditDeliveryPolicy",.response,policy))
	$$$ThrowOnError(response.Status)
	Quit response.ID
addSubscription(subscription)
	If $$$WriteProg Write !,?12,"Subscription: ",subscription.Name
	$$$ThrowOnError(##class(HS.Util.HubServices).InvokeHubService("HS.Push.WebServices","HS.Push.WebServicesClient","AddEditSubscriptionPolicy",.response,subscription))
	$$$ThrowOnError(response.Status)
	Quit response.ID
addSearchTerms(policy,basis,list)
	Set terms = ##class(HS.Push.WebServices.Containers.ListTermItems).%New()
	Set terms.PolicyId          = policy
	Set terms.SubscriptionBasis = "I"
	For i=1:1:$LL(list) {
		Do terms.Terms.Insert($LI(list,i))
		If $$$WriteProg Write !,?16,"Term: ",$LI(list,i)
	}
	$$$ThrowOnError(##class(HS.Util.HubServices).InvokeHubService("HS.Push.WebServices","HS.Push.WebServicesClient","AddEditSearchTerms",.sc,terms))
	$$$ThrowOnError(sc)
	Quit
}

/// Update productions
/// 
ClassMethod Productions()
{
	New $namespace
	Set $namespace = "HSLIB"
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:Productions - start"
	
	$$$ThrowOnError(##class(HS.Director).StopAll())
	
	;For ns = "HSREGISTRY","HSACCESS" {
	;	Set $namespace = ns
	;	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:Productions - ",$namespace
	;	$$$ThrowOnError(##class(HS.Util.Installer.Kit.Base).ProductionMods(,"xdata://Custom.Setup:"_$namespace,1))		
	;	If ns="HSACCESS" {
	;		$$$ThrowOnError(##class(HS.Util.Installer.Kit.Base).RemoveProductionItems(ns,"xdata://Custom.Setup:"_$namespace_"Remove"))
	;	}
	;}
	

	Set $namespace = "HSREGISTRY"
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:Productions - ",$namespace
	$$$ThrowOnError(##class(HS.Util.Installer.Kit.Base).ProductionMods(,"xdata://HS.Local.VA.HS.Registry.Setup:"_$namespace,1))		

	Set $namespace = "HSLIB"
	$$$ThrowOnError(##class(HS.Director).StartAll())
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:Productions - success"
}

/// Validate install
/// - did the custom code get loaded into HSCUSTOM?
/// - did the custom xslt get moved into the right locations
/// - TODO: is the license valid and cover the necessary functions
/// 
ClassMethod Validate()
{
	New $namespace
	Set $namespace = "HSCUSTOM"
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:Validate - start"

	If $$$WriteProg Write !,?4,"Custom code"
	;Set classes("Custom.Gateway.Access.QueryProcess") = ""
	Set classes("HS.Local.VA.HS.Registry.HS.IHE.XDSb.OnDemand.Operations") = ""
	;Set classes("Custom.IHE.XDSb.OnDemand.Process") = ""
	Set classes("HS.Local.VA.HS.Registry.Installer") = ""
	Set class="" For { Set class = $o(classes(class)) Quit:class=""
		If $$$WriteProg Write !,?8,class
		If '##class(%Dictionary.CompiledClass).%ExistsId(class) $$$ThrowStatus($$$ERROR(5001,"Class not found: "_class))
	}
	
	If $$$WriteProg Write !,?4,"Custom XSLT"
	Set install = $SYSTEM.Util.InstallDirectory()
	Set files($i(files)) = "/csp/xslt/SDA3/MPI/Custom/SDA-to-AddUpdateHub.xsl"
	Set files($i(files)) = "/csp/xslt/IHE/XDSb/Version1/Custom/ProcessRequest-to-DeliveryRequest.xsl"
	;Set files($i(files)) = "/csp/xslt/SDA3/Summaries/XML/Custom/SDA-to-XMLSummaryScan-Z-C62.xsl"
	;Set files($i(files)) = "/csp/xslt/SDA3/Summaries/XML/Custom/SDA-to-XMLSummaryScan-Z-CCDA-Encounter.xsl"
	;Set files($i(files)) = "/csp/xslt/SDA3/Summaries/XML/Custom/SDA-to-XMLSummaryScan-Z-CCDA-Patient.xsl"
	;Set files($i(files)) = "/csp/xslt/SDA3/Custom/Utility.xsl"
	;Set files($i(files)) = "/csp/xslt/SDA3/Custom/SDA-to-Z-C62.xsl"
	;Set files($i(files)) = "/csp/xslt/SDA3/Custom/SDA-to-Z-CCDA-Encounter.xsl"
	;Set files($i(files)) = "/csp/xslt/SDA3/Custom/cda.xsl"
	;Set files($i(files)) = "/csp/xslt/SDA3/Custom/cda_plain.css"
	
	Set i="" For { Set i=$o(files(i),1,file) Quit:i=""
		If $$$WriteProg Write !,?8,file
		Set full = ##class(%File).NormalizeFilename(install_"/"_file)
		If '##class(%File).Exists(full) $$$ThrowStatus($$$ERROR(5001,"File not found: "_full))
	}
	
	If $$$WriteProg Write !,"HS.Local.VA.HS.Registry.Setup:Validate - success"
}

/// convert namespace into HS.Gateway.Config ID
ClassMethod getGatewayID(namespace) As %String
{
	Set pattern = "%:"_namespace
	Set rs = ##class(%SQL.Statement).%ExecDirect(,"SELECT ID,Name From HS_Gateway.Config WHERE Name LIKE ?",pattern)
	If rs.%Next() Quit rs.ID
	$$$ThrowStatus($$$ERROR(5001,"Gateway not found: "_namespace))
}

/// convert service indo HS.Registry.Service.SOAP ID
ClassMethod getServiceID(serviceName) As %String
{
	Quit ##class(HS.Registry.Service.SOAP).IDForNameType(serviceName,"SOAP")
}

/// Setup customization to skip replace if no critial info changed (eg encounter start/stop date)
/// 
XData HSREGISTRY
{
<Items>
<Item Name="VA.IHE.XDSb.OnDemand.Operations" Category="" ClassName="HS.Local.VA.HS.Registry.HS.IHE.XDSb.OnDemand.Operations" PoolSize="1" Enabled="true" Foreground="false" InactivityTimeout="0" Comment="" LogTraceEvents="false" Schedule="">
<Setting Target="Host" Name="IssueOnDemandReplacements">1</Setting>
<Setting Target="Host" Name="OnDemandRepositoryName">HSACCCDA</Setting>
</Item>
<Item Name="VA.InactivePatientLoad.Process" Category="MVI" ClassName="HS.Local.VA.HS.Registry.InactivePatientLoad.Process" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="false" Schedule="">
  </Item>
  <Item Name="VA.InactivePatientLoad.Operation" Category="MVI" ClassName="HS.Local.VA.HS.Registry.InactivePatientLoad.Operation" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="false" Schedule="">
  </Item>
  <Item Name="VA.HS.IHE.XDSb.Registry.Operations" Category="MVI" ClassName="HS.Local.VA.HS.Registry.HS.IHE.XDSb.Registry.Operations" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="false" Schedule="">
    <Setting Target="Host" Name="OnDemandRepository">HSACCCDA</Setting>
    <Setting Target="Host" Name="InactivePatientTimeout">60</Setting>
  </Item>
  <Item Name="HS.IHE.XDSb.Registry.Services" Category="" ClassName="HS.IHE.XDSb.Registry.Services" PoolSize="0" Enabled="true" Foreground="false" Comment="" LogTraceEvents="false" Schedule="">
    <Setting Target="Host" Name="XDSbQueryTarget">HS.IHE.XDSb.Registry.Operations</Setting>
  </Item>
<Item Name="HS.Hub.Push.Evaluator" Category="" ClassName="HS.Hub.Push.Evaluator" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="false" Schedule="">
<Setting Target="Host" Name="XDSbOnDemandTarget">VA.IHE.XDSb.OnDemand.Operations</Setting>
</Item>
  <Item Name="VA.MVI.Operations" Category="MVI" ClassName="HS.Local.VA.HS.Registry.MVI.Operations" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="false" Schedule="">
    <Setting Target="Host" Name="TraceOperations">*ERRORSONLY*</Setting>
    <Setting Target="Host" Name="PatientSearch">VA.MVI.PDQv3.Consumer.Operations</Setting>
    <Setting Target="Host" Name="UseFacilityAsDefaultAA">1</Setting>
    <Setting Target="Host" Name="ArchiveIO">0</Setting>
  </Item>
  <Item Name="EnsLib.File.PassthroughService" Category="" ClassName="EnsLib.File.PassthroughService" PoolSize="1" Enabled="false" Foreground="false" Comment="" LogTraceEvents="false" Schedule="">
  </Item>
  <Item Name="VA.MVI.PDQv3.Consumer.Operations.DFN" Category="MVI" ClassName="HS.Local.VA.HS.Registry.HS.IHE.PDQv3.Consumer.Operations" PoolSize="1" Enabled="true" Foreground="false" Comment="retrieve with register interest" LogTraceEvents="false" Schedule="">
    <Setting Target="Host" Name="ArchiveIO">1</Setting>
    <Setting Target="Host" Name="TraceOperations">*FULL*</Setting>
  </Item>
  <Item Name="VA.MVI.PDQv3.Consumer.Operations" Category="MVI" ClassName="HS.Local.VA.HS.Registry.HS.IHE.PDQv3.Consumer.Operations" PoolSize="1" Enabled="true" Foreground="false" Comment="patient search" LogTraceEvents="false" Schedule="">
    <Setting Target="Host" Name="ArchiveIO">1</Setting>
    <Setting Target="Host" Name="Device">200HSEP</Setting>
    <Setting Target="Host" Name="ServiceName">MVI.PDQv3.Consumer</Setting>
    <Setting Target="Host" Name="TraceOperations">*FULL*</Setting>
    <Setting Target="Host" Name="TransformPDQToPatientSearch">IHE/PDQ/Version1/Custom/vaPRPAIN201306UVToPatientSearchResponse.xsl</Setting>
    <Setting Target="Host" Name="TransformPatientSearchToPDQ">IHE/PDQ/Version1/Custom/vaPatientSearchToPRPAIN201305UV.xsl</Setting>
  </Item>
  <Item Name="VA.HS.IHE.PIX.Consumer.Process" Category="MVI" ClassName="HS.Local.VA.HS.Registry.HS.IHE.PIX.Consumer.Process" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="true" Schedule="">
    <Setting Target="Host" Name="TraceOperations">*FULL*</Setting>
    <Setting Target="Host" Name="HL7ProcessingID">T</Setting>
    <Setting Target="Host" Name="PartnerOperation">VA.MVI.ACK.Operation</Setting>
    <Setting Target="Host" Name="VAStation">200HSEP</Setting>
  </Item>
  <Item Name="VA.ESR.Operations" Category="MVI" ClassName="HS.Local.VA.HS.Registry.ESR.Operations" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="true" Schedule="">
    <Setting Target="Host" Name="ServiceName">VA.ESR</Setting>
    <Setting Target="Adapter" Name="Credentials"></Setting>
    <Setting Target="Adapter" Name="SOAPCredentials"></Setting>
    <Setting Target="Host" Name="ArchiveIO">1</Setting>
  </Item>
  <Item Name="VA.MVI.ACK.Operation" Category="MVI" ClassName="EnsLib.HL7.Operation.TCPOperation" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="true" Schedule="">
    <Setting Target="Adapter" Name="IPAddress">ecptest-dev.DNS           </Setting>
    <Setting Target="Adapter" Name="Port">PORT</Setting>
    <Setting Target="Adapter" Name="ConnectTimeout">20</Setting>
    <Setting Target="Host" Name="Separators"><![CDATA[^~|\&]]></Setting>
    <Setting Target="Host" Name="ArchiveIO">1</Setting>
    <Setting Target="Adapter" Name="RegistryID"></Setting>
    <Setting Target="Adapter" Name="StayConnected">120</Setting>
  </Item>
  <Item Name="VA.HS.IHE.PIXv2.Consumer.Services" Category="MVI" ClassName="HS.Local.VA.HS.Registry.HS.IHE.PIXv2.Consumer.Services" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="true" Schedule="">
    <Setting Target="Host" Name="MessageSchemaCategory">VA.MVI.2.5</Setting>
    <Setting Target="Host" Name="TargetConfigNames">VA.HS.IHE.PIX.Consumer.Process</Setting>
    <Setting Target="Host" Name="AckMode">Immed</Setting>
    <Setting Target="Adapter" Name="StayConnected">-1</Setting>
    <Setting Target="Adapter" Name="JobPerConnection">1</Setting>
    <Setting Target="Adapter" Name="Port">PORT</Setting>
    <Setting Target="Host" Name="ArchiveIO">1</Setting>
    <Setting Target="Host" Name="LocalFacilityApplication">@:@</Setting>
  </Item>
  <Item Name="HS.IHE.XDSb.Registry.Operations" Category="" ClassName="HS.IHE.XDSb.Registry.Operations" PoolSize="1" Enabled="true" Foreground="false" Comment="" LogTraceEvents="false" Schedule="">
    <Setting Target="Host" Name="OnDemandRepository">HSACCCDA</Setting>
  </Item>
</Items>
}

/// Setup customization to work around document filter bug (and eventually support user supplied parameters via search-and-fetch)
/// 
XData HSACCESS
{
<Items>
<Item Name="Custom.Gateway.Access.QueryProcess" Category="" ClassName="Custom.Gateway.Access.QueryProcess" PoolSize="0" Enabled="true" Foreground="false" InactivityTimeout="0" Comment="" LogTraceEvents="false" Schedule="">
</Item>
<Item Name="Custom.IHE.XDSb.OnDemand.Process" Category="" ClassName="Custom.IHE.XDSb.OnDemand.Process" PoolSize="0" Enabled="true" Foreground="false" InactivityTimeout="0" Comment="" LogTraceEvents="false" Schedule="">
<Setting Target="Host" Name="QueryProcess">Custom.Gateway.Access.QueryProcess</Setting>
</Item>
<Item Name="HS.IHE.XDSb.Repository.Services" Category="" ClassName="HS.IHE.XDSb.Repository.Services" PoolSize="0" Enabled="true" Foreground="false" InactivityTimeout="0" Comment="" LogTraceEvents="false" Schedule="">
<Setting Target="Host" Name="XDSbRetrieveTarget">Custom.IHE.XDSb.OnDemand.Process</Setting>
</Item>
</Items>
}

XData HSACCESSRemove
{
<Items>
<Item Name='HS.Gateway.Access.QueryProcess' ClassName='HS.Gateway.Access.QueryProcess'/>
<Item Name='HS.IHE.XDSb.OnDemand.Process' ClassName='HS.IHE.XDSb.OnDemand.Process'/>
</Items>
}

ClassMethod Reset()
{
	New $namespace

	Set $namespace = "HSLIB"
	Do ##class(HS.Director).StopAll()
	
	Set $namespace = "HSREGISTRY"
	Do ##class(HS.Util.Installer.Hub).Reset()
	
	Set $namespace = "HSEDGE1"
	Do ##class(HS.Util.Installer.EdgeGateway).Reset()

	Set $namespace = "HSLIB"
	Do ##class(HS.Director).StartAll()
}

}
